#!/usr/bin/python
import codecs
import glob
import os

from pybtex.database.input import bibtex
from pybtex.style.names.plain import NameStyle
from pybtex.style.formatting.unsrt import Style

BIBTEX_FILE = "mchro.bib"
OUTPUT_FILE = "pubs.rst"
ALLPAPERS_DIR = "allpapers"
OUTPUTPAPERS_DIR = "generated/papers"

outfile = codecs.open(OUTPUT_FILE, "w", "utf-8")

namestyle = NameStyle()

parser = bibtex.Parser()
bib_data = parser.parse_file(BIBTEX_FILE)
bib_data.entries.keys()

self_entries = [x for x in bib_data.entries.viewvalues() if 
    'Self' in x.fields.get('tags', '') and
    x.type != 'unpublished']

def strip_quotes(x):
    if x != "" and x[0] == '{' and x[-1] == "}":
        return x[1:-1]
    return x

def get_entry_link(entry):
    files = glob.glob('generated/papers/' + entry.key + ".pdf")
    if files != []:
        return "papers/" + os.path.basename(files[0])
    return None


def entry_to_rst(entry):
    key = entry.key
    author_entries = entry.persons['author']
    title = strip_quotes(entry.fields['title'])
    year = entry.fields['year']
    month = strip_quotes(entry.fields.get('month', ''))
    authors = map(namestyle.format, author_entries)
    authors = u", ".join(map(lambda x: x.format().plaintext(), authors))
    authors = authors.replace(u"<nbsp>", u" ")
    pdflink = get_entry_link(entry)
    haslink = pdflink and "_" or ""

    rst_link_replace = u"\n.. |{key}| replace:: **{title}**\n".format(**locals())
    if pdflink:
        rst_link_replace = rst_link_replace + u".. _{key}: {pdflink}\n".format(**locals())

    if entry.type == "inproceedings":
        booktitle = strip_quotes(entry.fields.get("booktitle", ""))
        publisher = entry.fields.get("publisher", "") and (
            u", " + strip_quotes(entry.fields.get("publisher", ""))) or ""
        pages = entry.fields.get("pages", "") and (
                u", pages " + entry.fields.get("pages", "")) or ""
        return u"""
|{key}|{haslink}
  {authors} |br|
  In {booktitle}{pages}, {year}{publisher}.

{rst_link_replace}
""".format(**locals())
    elif entry.type == "article":
        journal = strip_quotes(entry.fields.get("journal", ""))
        #publisher = entry.fields.get("publisher", "") and (
        #    u", " + strip_quotes(entry.fields.get("publisher", ""))) or ""
        pages = entry.fields.get("pages", "") and (
                u", pages " + entry.fields.get("pages", "")) or ""
        return u"""
|{key}|{haslink}
  {authors} |br|
  In {journal}{pages}, {year}.

{rst_link_replace}
""".format(**locals())
    elif entry.type == "misc":
        howpublished = strip_quotes(entry.fields.get("howpublished", ""))
        return u"""
|{key}|{haslink}
  {authors} |br|
  In {howpublished}, {year}.

{rst_link_replace}
""".format(**locals())
    elif entry.type == "mastersthesis":
        school = strip_quotes(entry.fields.get("school", ""))
        return u"""
|{key}|{haslink}
  {authors} |br|
  Masters thesis, {school}, {month} {year}.

{rst_link_replace}
""".format(**locals())
    elif entry.type == "phdthesis":
        school = strip_quotes(entry.fields.get("school", ""))
        return u"""
|{key}|{haslink}
  {authors} |br|
  PhD thesis, {school}, {month} {year}.

{rst_link_replace}
""".format(**locals())

    return u"""
|{key}|{haslink}
  {authors}, {year}

{rst_link_replace}
""".format(**locals())

self_entries.sort(key=lambda e: e.fields.get('year', 0), reverse=True)

#entries = ((key, bib_data.entries[key]) for key in citations)
#entries = map(lambda x: ("", x), self_entries)
#print list(style.format_entries(entries))

outfile.write("""
.. |br| raw:: html

   <br />

""")

for e in self_entries:
    outfile.write(entry_to_rst(e))

